@@ -17,7 +17,7 @@ from ..io.resources import CDN_DIST
 from ..layout import Row, Tabs
 from ..pane.image import ImageBase
 from ..viewable import Viewable
-from ..widgets.base import Widget
+from ..widgets.base import WidgetBase
 from ..widgets.button import Button
 from ..widgets.input import FileInput, TextInput
 from .feed import CallbackState, ChatFeed
@@ -110,7 +110,7 @@ class ChatInterface(ChatFeed):
     user = param.String(default="User", doc="""
         Name of the ChatInterface user.""")
 
-    widgets = param.ClassSelector(class_=(Widget, list), allow_refs=False, doc="""
+    widgets = param.ClassSelector(class_=(WidgetBase, list), allow_refs=False, doc="""
         Widgets to use for the input. If not provided, defaults to
         `[TextInput]`.""")
 
@@ -251,7 +251,7 @@ class ChatInterface(ChatFeed):
             )
 
         widgets = self.widgets
-        if isinstance(self.widgets, Widget):
+        if isinstance(self.widgets, WidgetBase):
             widgets = [self.widgets]
 
         self._widgets = {}
@@ -413,8 +413,7 @@ class ChatInterface(ChatFeed):
                     mime_type=active_widget.mime_type,
                     file_name=active_widget.filename,
                 )
-            # don't use isinstance here; TextAreaInput subclasses TextInput
-            if type(active_widget) is TextInput or self.reset_on_send:
+            if isinstance(value, TextInput) or self.reset_on_send:
                 updates = {"value": ""}
                 if hasattr(active_widget, "value_input"):
                     updates["value_input"] = ""
@@ -540,7 +539,7 @@ class ChatInterface(ChatFeed):
             self._reset_button_data()
 
     @property
-    def active_widget(self) -> Widget:
+    def active_widget(self) -> WidgetBase:
         """
         The currently active widget.
 
@@ -585,7 +584,8 @@ class ChatInterface(ChatFeed):
         messages: list[ChatMessage],
         role_names: dict[str, str | list[str]] | None = None,
         default_role: str | None = "assistant",
-        custom_serializer: Callable = None
+        custom_serializer: Callable = None,
+        **serialize_kwargs
     ) -> list[dict[str, Any]]:
         """
         Exports the chat log for use with transformers.
@@ -607,6 +607,8 @@ class ChatInterface(ChatFeed):
             A custom function to format the ChatMessage's object. The function must
             accept one positional argument, the ChatMessage object, and return a string.
             If not provided, uses the serialize method on ChatMessage.
+        serialize_kwargs : dict
+            Additional keyword arguments to pass to the serializer.
 
         Returns
         -------
@@ -617,7 +619,8 @@ class ChatInterface(ChatFeed):
                 "user": [self.user],
                 "assistant": [self.callback_user],
             }
-        return super()._serialize_for_transformers(messages, role_names, default_role, custom_serializer)
+        return super()._serialize_for_transformers(
+            messages, role_names, default_role, custom_serializer, **serialize_kwargs)
 
     @param.depends("_callback_state", watch=True)
     async def _update_input_disabled(self):
@@ -718,7 +721,7 @@ class ChatInterface(ChatFeed):
         -------
         The message that was updated.
         """
-        if not isinstance(value, ChatMessage):
+        if not isinstance(value, ChatMessage) and not message:
             # ChatMessage cannot set user or avatar when explicitly streaming
             # so only set to the default when not a ChatMessage
             user = user or self.user
