@@ -5,16 +5,14 @@ The message module provides a low-level API for rendering chat messages.
 from __future__ import annotations
 
 import datetime
-import re
 
 from contextlib import ExitStack
 from dataclasses import dataclass
 from functools import partial
 from io import BytesIO
 from tempfile import NamedTemporaryFile
-from textwrap import indent
 from typing import (
-    TYPE_CHECKING, Any, Callable, ClassVar, Iterable, Optional, Union,
+    TYPE_CHECKING, Any, Callable, ClassVar, Optional, Union,
 )
 from zoneinfo import ZoneInfo
 
@@ -23,7 +21,7 @@ import param
 from ..io.resources import CDN_DIST, get_dist_path
 from ..io.state import state
 from ..layout import Column, Row
-from ..pane.base import PaneBase, ReplacementPane, panel as _panel
+from ..pane.base import Pane, ReplacementPane, panel as _panel
 from ..pane.image import (
     PDF, FileBase, Image, ImageBase,
 )
@@ -35,6 +33,9 @@ from ..param import ParamFunction
 from ..viewable import Viewable
 from ..widgets.base import Widget
 from .icon import ChatCopyIcon, ChatReactionIcons
+from .utils import (
+    avatar_lookup, build_avatar_pane, serialize_recursively, stream_to,
+)
 
 if TYPE_CHECKING:
     from bokeh.document import Document
@@ -49,6 +50,7 @@ ASSISTANT_LOGO = "🤖"
 SYSTEM_LOGO = "⚙️"
 ERROR_LOGO = "❌"
 HELP_LOGO = "❓"
+INPUT_LOGO = "❗"
 GPT_3_LOGO = "{dist_path}assets/logo/gpt-3.svg"
 GPT_4_LOGO = "{dist_path}assets/logo/gpt-4.svg"
 WOLFRAM_LOGO = "{dist_path}assets/logo/wolfram.svg"
@@ -78,6 +80,7 @@ DEFAULT_AVATARS = {
     "exception": ERROR_LOGO,
     "error": ERROR_LOGO,
     "help": HELP_LOGO,
+    "input": INPUT_LOGO,
     # Human
     "adult": "🧑",
     "baby": "👶",
@@ -132,11 +135,13 @@ class _FileInputMessage:
     mime_type: str
 
 
-class ChatMessage(PaneBase):
+class ChatMessage(Pane):
     """
-    A widget for displaying chat messages with support for various content types.
+    Renders another component as a chat message with an associated user
+    and avatar with support for various content types.
 
     This widget provides a structured view of chat messages, including features like:
+
     - Displaying user avatars, which can be text, emoji, or images.
     - Showing the user's name.
     - Displaying the message timestamp in a customizable format.
@@ -176,7 +181,7 @@ class ChatMessage(PaneBase):
     header_objects = param.List(doc="""
         A list of objects to display in the row of the header of the message.""")
 
-    max_width = param.Integer(default=1200, bounds=(0, None))
+    max_width = param.Integer(default=1200, bounds=(0, None), allow_None=True)
 
     object = param.Parameter(allow_refs=False, doc="""
         The message contents. Can be any Python object that panel can display.""")
@@ -249,7 +254,7 @@ class ChatMessage(PaneBase):
 
         reaction_icons = params.get("reaction_icons", {"favorite": "heart"})
         if isinstance(reaction_icons, dict):
-            params["reaction_icons"] = ChatReactionIcons(options=reaction_icons)
+            params["reaction_icons"] = ChatReactionIcons(options=reaction_icons, default_layout=Row, sizing_mode=None)
         self._internal = True
         super().__init__(object=object, **params)
         self.chat_copy_icon = ChatCopyIcon(
@@ -261,20 +266,14 @@ class ChatMessage(PaneBase):
         self._build_layout()
 
     def _build_layout(self):
-        self._activity_dot = HTML(
-            "●",
-            css_classes=["activity-dot"],
-            visible=self.param.show_activity_dot,
-            stylesheets=self._stylesheets + self.param.stylesheets.rx(),
-        )
         self._left_col = left_col = Column(
             self._render_avatar(),
             max_width=60,
             height=100,
             css_classes=["left"],
-            stylesheets=self._stylesheets + self.param.stylesheets.rx(),
             visible=self.param.show_avatar,
             sizing_mode=None,
+            stylesheets=self._stylesheets + self.param.stylesheets.rx(),
         )
         self.param.watch(self._update_avatar_pane, "avatar")
 
@@ -282,7 +281,6 @@ class ChatMessage(PaneBase):
         self._update_chat_copy_icon()
         self._center_row = Row(
             self._object_panel,
-            self._render_reaction_icons(),
             css_classes=["center"],
             stylesheets=self._stylesheets + self.param.stylesheets.rx(),
             sizing_mode=None
@@ -291,42 +289,65 @@ class ChatMessage(PaneBase):
         self.param.watch(self._update_reaction_icons, "reaction_icons")
 
         self._user_html = HTML(
-            self.param.user, height=20, css_classes=["name"],
-            visible=self.param.show_user, stylesheets=self._stylesheets,
+            self.param.user, height=20,
+            css_classes=["name"],
+            visible=self.param.show_user,
+            sizing_mode=None,
         )
 
-        header_objects = (
-            [self._user_html] +
-            self.param.header_objects.rx() +
-            [self.chat_copy_icon, self._activity_dot]
+        self._activity_dot = HTML(
+            "●",
+            css_classes=["activity-dot"],
+            margin=(5, 0),
+            sizing_mode=None,
+            visible=self.param.show_activity_dot,
         )
-        header_row = Row(
-            objects=header_objects,
+
+        meta_row = Row(
+            self._user_html,
+            self._activity_dot,
+            css_classes=["meta"],
             stylesheets=self._stylesheets + self.param.stylesheets.rx(),
+        )
+
+        header_col = Column(
+            objects=self.param.header_objects.rx(),
             sizing_mode="stretch_width",
-            css_classes=["header"]
+            css_classes=["header"],
+            stylesheets=self._stylesheets + self.param.stylesheets.rx(),
+        )
+
+        footer_col = Column(
+            objects=self.param.footer_objects.rx(),
+            sizing_mode="stretch_width",
+            css_classes=["footer"],
+            stylesheets=self._stylesheets + self.param.stylesheets.rx(),
         )
 
         self._timestamp_html = HTML(
             self.param.timestamp.rx().strftime(self.param.timestamp_format),
             css_classes=["timestamp"],
-            visible=self.param.show_timestamp
+            visible=self.param.show_timestamp,
         )
 
-        footer_col = Column(
-            objects=self.param.footer_objects.rx() + [self._timestamp_html],
-            stylesheets=self._stylesheets + self.param.stylesheets.rx(),
+        self._icons_row = Row(
+            self.chat_copy_icon,
+            self._render_reaction_icons(),
+            css_classes=["icons"],
             sizing_mode="stretch_width",
-            css_classes=["footer"],
+            stylesheets=self._stylesheets + self.param.stylesheets.rx(),
         )
 
         self._right_col = right_col = Column(
-            header_row,
+            meta_row,
+            header_col,
             self._center_row,
             footer_col,
+            self._icons_row,
+            self._timestamp_html,
             css_classes=["right"],
+            sizing_mode=None,
             stylesheets=self._stylesheets + self.param.stylesheets.rx(),
-            sizing_mode=None
         )
         viewable_params = {
             p: self.param[p] for p in self.param if p in Viewable.param
@@ -335,75 +356,11 @@ class ChatMessage(PaneBase):
         viewable_params['stylesheets'] = self._stylesheets + self.param.stylesheets.rx()
         self._composite = Row(left_col, right_col, **viewable_params)
 
-    def _get_obj_label(self, obj):
-        """
-        Get the label for the object; defaults to specified object name;
-        if unspecified, defaults to the type name.
-        """
-        label = obj.name
-        type_name = type(obj).__name__
-        # If the name is just type + ID, simply use type
-        # e.g. Column10241 -> Column
-        if label.startswith(type_name) or not label:
-            label = type_name
-        return label
-
-    def _serialize_recursively(
-        self,
-        obj: Any,
-        prefix_with_viewable_label: bool = True,
-        prefix_with_container_label: bool = True
-    ) -> str:
-        """
-        Recursively serialize the object to a string.
-        """
-        if isinstance(obj, Iterable) and not isinstance(obj, str):
-            content = tuple(
-                self._serialize_recursively(
-                    o,
-                    prefix_with_viewable_label=prefix_with_viewable_label,
-                    prefix_with_container_label=prefix_with_container_label
-                ) for o in obj
-            )
-            if prefix_with_container_label:
-                if len(content) == 1:
-                    return f"{self._get_obj_label(obj)}({content[0]})"
-                else:
-                    indented_content = indent(",\n".join(content), prefix=" " * 4)
-                    # outputs like:
-                    # Row(
-                    #   1,
-                    #   "str",
-                    # )
-                    return f"{self._get_obj_label(obj)}(\n{indented_content}\n)"
-            else:
-                # outputs like:
-                # (1, "str")
-                return f"({', '.join(content)})"
-
-        string = obj
-        if hasattr(obj, "value"):
-            string = obj.value
-        elif hasattr(obj, "object"):
-            string = obj.object
-
-        if hasattr(string, "decode") or isinstance(string, BytesIO):
-            self.param.warning(
-                f"Serializing byte-like objects are not supported yet; "
-                f"using the label of the object as a placeholder for {obj}"
-            )
-            return self._get_obj_label(obj)
-
-        if prefix_with_viewable_label and isinstance(obj, Viewable):
-            label = self._get_obj_label(obj)
-            string = f"{label}={string!r}"
-        return string
-
     def __str__(self) -> str:
         """
         Serialize the message object to a string.
         """
-        return self.serialize()
+        return str(self.serialize())
 
     @property
     def _synced_params(self) -> list[str]:
@@ -423,31 +380,6 @@ class ChatMessage(PaneBase):
         self._models[ref] = (model, parent)
         return model
 
-    @staticmethod
-    def _to_alpha_numeric(user: str) -> str:
-        """
-        Convert the user name to an alpha numeric string,
-        removing all non-alphanumeric characters.
-        """
-        return re.sub(r"\W+", "", user).lower()
-
-    def _avatar_lookup(self, user: str) -> Avatar:
-        """
-        Lookup the avatar for the user.
-        """
-        alpha_numeric_key = self._to_alpha_numeric(user)
-        # always use the default first
-        updated_avatars = DEFAULT_AVATARS.copy()
-        # update with the user input
-        updated_avatars.update(self.default_avatars)
-        # correct the keys to be alpha numeric
-        updated_avatars = {
-            self._to_alpha_numeric(key): value for key, value in updated_avatars.items()
-        }
-
-        # now lookup the avatar
-        return updated_avatars.get(alpha_numeric_key, self.avatar).format(dist_path=CDN_DIST)
-
     def _select_renderer(
         self,
         contents: Any,
@@ -573,7 +505,7 @@ class ChatMessage(PaneBase):
                 pass
         else:
             if isinstance(old, Markdown) and isinstance(value, str):
-                self._set_params(old, object=value)
+                self._set_params(old, enable_streaming=True, object=value)
                 return old
             object_panel = _panel(value)
 
@@ -593,26 +525,9 @@ class ChatMessage(PaneBase):
         if not avatar and self.user:
             avatar = self.user[0]
 
-        avatar_params = {'css_classes': ["avatar"]}
-        if isinstance(avatar, ImageBase):
-            avatar_pane = avatar
-            avatar_params['css_classes'] = (
-                avatar_params.get('css_classes', []) +
-                avatar_pane.css_classes
-            )
-            avatar_params.update(width=35, height=35)
-            avatar_pane.param.update(avatar_params)
-        elif not isinstance(avatar, (BytesIO, bytes)) and len(avatar) == 1:
-            # single character
-            avatar_pane = HTML(avatar, **avatar_params)
-        else:
-            try:
-                avatar_pane = Image(
-                    avatar, width=35, height=35, **avatar_params
-                )
-            except ValueError:
-                # likely an emoji
-                avatar_pane = HTML(avatar, **avatar_params)
+        avatar_pane = build_avatar_pane(
+            avatar, css_classes=["avatar"], height=35, width=35
+        )
         return avatar_pane
 
     def _update_avatar_pane(self, event=None):
@@ -639,7 +554,7 @@ class ChatMessage(PaneBase):
         return reaction_icons
 
     def _update_reaction_icons(self, _):
-        self._center_row[1] = self._render_reaction_icons()
+        self._icons_row[-1] = self._render_reaction_icons()
 
     def _update(self, ref, old_models):
         """
@@ -666,7 +581,12 @@ class ChatMessage(PaneBase):
         if self.avatar_lookup:
             self.avatar = self.avatar_lookup(self.user)
         else:
-            self.avatar = self._avatar_lookup(self.user)
+            self.avatar = avatar_lookup(
+                self.user,
+                self.avatar,
+                self.default_avatars,
+                DEFAULT_AVATARS,
+            )
 
     def _update_chat_copy_icon(self):
         object_panel = self._object_panel
@@ -703,33 +623,7 @@ class ChatMessage(PaneBase):
         replace: bool (default=False)
             Whether to replace the existing text.
         """
-        i = -1
-        parent_panel = None
-        object_panel = self
-        attr = "object"
-        obj = self.object
-        if obj is None:
-            obj = ""
-
-        while not isinstance(obj, str) or isinstance(object_panel, ImageBase):
-            object_panel = obj
-            if hasattr(obj, "objects"):
-                parent_panel = obj
-                attr = "objects"
-                obj = obj.objects[i]
-                i = -1
-            elif hasattr(obj, "object"):
-                attr = "object"
-                obj = obj.object
-            elif hasattr(obj, "value"):
-                attr = "value"
-                obj = obj.value
-            elif parent_panel is not None:
-                obj = parent_panel
-                parent_panel = None
-                i -= 1
-        contents = token if replace else obj + token
-        setattr(object_panel, attr, contents)
+        stream_to(obj=self.object, token=token, replace=replace, object_panel=self)
 
     def update(
         self,
@@ -787,8 +681,6 @@ class ChatMessage(PaneBase):
 
         Arguments
         ---------
-        obj : Any
-            The object to format.
         prefix_with_viewable_label : bool
             Whether to include the name of the Viewable, or type
             of the viewable if no name is specified.
@@ -801,7 +693,7 @@ class ChatMessage(PaneBase):
         str
             The serialized string.
         """
-        return self._serialize_recursively(
+        return serialize_recursively(
             self.object,
             prefix_with_viewable_label=prefix_with_viewable_label,
             prefix_with_container_label=prefix_with_container_label,
