@@ -12,6 +12,7 @@ from types import FunctionType, MethodType
 from typing import (
     TYPE_CHECKING, Any, Callable, Mapping,
 )
+from urllib.parse import urljoin
 
 import bokeh.command.util
 
@@ -121,8 +122,30 @@ class Application(BkApplication):
             handler._on_session_destroyed = _on_session_destroyed
         super().add(handler)
 
+    def _set_session_prefix(self, doc):
+        session_context = doc.session_context
+        if not (session_context and session_context.server_context):
+            return
+        request = session_context.request
+        app_context = session_context.server_context.application_context
+        prefix = request.uri.replace(app_context._url, '')
+        if not prefix.endswith('/'):
+            prefix += '/'
+        base_url = urljoin('/', prefix)
+        rel_path = '/'.join(['..'] * app_context._url.strip('/').count('/'))
+
+        # Handle autoload.js absolute paths
+        abs_url = request.arguments.get('bokeh-absolute-url')
+        if abs_url:
+            rel_path = abs_url[0].decode('utf-8').replace(app_context._url, '')
+
+        with set_curdoc(doc):
+            state.base_url = base_url
+            state.rel_path = rel_path
+
     def initialize_document(self, doc):
         logger.info(LOG_SESSION_LAUNCHING, id(doc))
+        self._set_session_prefix(doc)
         super().initialize_document(doc)
         if doc in state._templates and doc not in state._templates[doc]._documents:
             template = state._templates[doc]
