@@ -17,9 +17,7 @@ except Exception:
     diskcache = None
 diskcache_available = pytest.mark.skipif(diskcache is None, reason="requires diskcache")
 
-from panel.io.cache import (
-    _find_hash_func, _generate_hash, cache, is_equal,
-)
+from panel.io.cache import _generate_hash, cache, is_equal
 from panel.io.state import set_curdoc, state
 from panel.tests.util import serve_and_wait
 
@@ -28,7 +26,7 @@ from panel.tests.util import serve_and_wait
 ################
 
 def hashes_equal(v1, v2):
-    a, b = _find_hash_func(v1)(v1), _find_hash_func(v2)(v2)
+    a, b = _generate_hash(v1), _generate_hash(v2)
     return a == b
 
 def test_str_hash():
@@ -52,6 +50,11 @@ def test_none_hash():
     assert hashes_equal(None, None)
     assert not hashes_equal(None, False)
 
+def test_object_hash():
+    obj1, obj2 = object(), object()
+    assert hashes_equal(obj1, obj1)
+    assert not hashes_equal(obj1, obj2)
+
 def test_bytes_hash():
     assert hashes_equal(b'0', b'0')
     assert not hashes_equal(b'0', b'1')
@@ -70,10 +73,11 @@ def test_list_hash():
     assert not hashes_equal([0], [1])
     assert not hashes_equal(['a', ['b']], ['a', ['c']])
 
+def test_list_hash_recursive():
     # Recursion
     l = [0]
     l.append(l)
-    assert hashes_equal(l, list(l))
+    assert hashes_equal(list(l), list(l))
 
 def test_tuple_hash():
     assert hashes_equal((0,), (0,))
@@ -88,10 +92,10 @@ def test_dict_hash():
     assert not hashes_equal({'a': 0}, {'a': 1})
     assert not hashes_equal({'a': {'b': 0}}, {'a': {'b': 1}})
 
-    # Recursion
+def test_dict_hash_recursive():
     d = {'a': {}}
     d['a'] = d
-    assert hashes_equal(d, dict(d))
+    assert hashes_equal(dict(d), dict(d))
 
 def test_stringio_hash():
     sio1, sio2 = io.StringIO(), io.StringIO()
@@ -151,6 +155,34 @@ def test_series_hash():
     series2.iloc[0] = 3.14
     assert not hashes_equal(series1, series2)
 
+def test_polars_dataframe_hash():
+    pl = pytest.importorskip("polars")
+    data = {
+        "A": [0.0, 1.0, 2.0, 3.0, 4.0],
+        "B": [0.0, 1.0, 0.0, 1.0, 0.0],
+        "C": ["foo1", "foo2", "foo3", "foo4", "foo5"],
+    }
+    # DataFrame
+    df1, df2 = pl.DataFrame(data), pl.DataFrame(data)
+    assert hashes_equal(df1, df2)
+    df2 = df2.with_columns(A=pl.col("A").sort(descending=True))
+    assert not hashes_equal(df1, df2)
+
+    # Lazy DataFrame
+    df1, df2 = pl.LazyFrame(data), pl.LazyFrame(data)
+    assert hashes_equal(df1, df2)
+    df2 = df2.with_columns(A=pl.col("A").sort(descending=True))
+    assert not hashes_equal(df1, df2)
+
+def test_polars_series_hash():
+    pl = pytest.importorskip("polars")
+    ser1 = pl.Series([0.0, 1.0, 2.0, 3.0, 4.0])
+    ser2 = ser1.clone()
+
+    assert hashes_equal(ser1, ser2)
+    ser2 = ser2.replace(0.0, 3.14)
+    assert not hashes_equal(ser1, ser2)
+
 def test_ufunc_hash():
     assert hashes_equal(np.absolute, np.absolute)
     assert not hashes_equal(np.sin, np.cos)
