@@ -29,6 +29,7 @@ from ..config import config
 from ..io import state
 from ..io.resources import CDN_DIST
 from ..layout import Column, Panel, Row
+from ..models.datetime_slider import DatetimeSlider as _BkDatetimeSlider
 from ..util import (
     datetime_as_utctimestamp, edit_readonly, param_reprs, value_as_date,
     value_as_datetime,
@@ -319,6 +320,36 @@ class DateSlider(_SliderBase):
         return msg
 
 
+class DatetimeSlider(DateSlider):
+    """
+    The DatetimeSlider widget allows selecting a value within a set of
+    bounds using a slider. Supports datetime.date, datetime.datetime
+    and np.datetime64 values. The step size is fixed at 1 minute.
+
+    Reference: https://panel.holoviz.org/reference/widgets/DatetimeSlider.html
+
+    :Example:
+
+    >>> import datetime as dt
+    >>> DatetimeSlider(
+    ...     value=dt.datetime(2025, 1, 1),
+    ...     start=dt.datetime(2025, 1, 1),
+    ...     end=dt.datetime(2025, 1, 7),
+    ...     name="A datetime value"
+    ... )
+    """
+
+    as_datetime = param.Boolean(default=True, readonly=True, doc="""
+        Whether to store the date as a datetime.""")
+
+    step = param.Number(default=60, bounds=(1, None), doc="""
+        The step size in seconds. Default is 1 minute, i.e 60 seconds.""")
+
+    _property_conversion = staticmethod(value_as_datetime)
+
+    _widget_type: ClassVar[type[Model]] = _BkDatetimeSlider
+
+
 class DiscreteSlider(CompositeWidget, _SliderBase):
     """
     The DiscreteSlider widget allows selecting a value from a discrete
